/***********************/
/* "FTInstrumental4.c" */
/***********************/

/***** INTERFACE *****/

/*** Function Prototypes (uses) ***/
void frenel(double x, double *s, double *c);	/* in "fresnel.c" */

/*** Function Prototypes (public) ***/
void FTKalpha1(double *re,double *im,double x,double g1);
	/* in "FTInstrumental4.c" */
void FTKalpha2(double *re,double *im,double x,double rho,double x2,double dg);
	/* in "FTInstrumental4.c" */
void FTGauss(double *re,double *im,double x,double gG);
	/* in "FTInstrumental4.c" */
void FTUpperAxial(double *re,double *im,double xi,double PhiA);
	/* in "FTInstrumental4.c" */
void FTLowerAxial(double *re,double *im,double xi,double PhiA);
	/* in "FTInstrumental4.c" */
void FTFlatSpecimen(double *re,double *im,double xi,double Phi);
	/* in "FTInstrumental4.c" */
void FTTransparency(double *re,double *im,double xi,double invMuR);
	/* in "FTInstrumental4.c" */
void FTReceivingSlit(double *re, double *im, double xi, double rr);
    /* in "FTInstrumental4.c" */


/***** IMPLEMENTATION *****/

#include <math.h>

#define PI 3.141592653589793238462643383279

static double sqr(double x)
{
	return x*x;
}

void FTKalpha1(double *re,double *im,double xi,double g1)
{
	double r;
	if (g1==0.0) {
		*re = 1.0;
		*im = 0.0;
	} else {
		if (xi==0.0) {
			*re = 1.0;
			*im = 0.0;
		} else {
			r=exp(-2*PI*xi*g1);
			*re = r;
			*im = 0.0;
		}
	}
}

void FTKalpha2(double *re,double *im,double xi,double rho,double x2,double dg)
{
	double r;
	if (rho==0.0) {
		*re = 1.0;
		*im = 0.0;
	} else {
		if (xi==0.0) {
			*re = 1.0;
			*im = 0.0;
		} else {
			r=rho*exp(-2*PI*xi*dg);
			*re = 1 - rho + r * cos(2*PI*xi*x2);
			*im = r * sin(2*PI*xi*x2);
		}
	}
}

void FTGauss(double *re,double *im,double xi,double gG)
{
	if (gG==0.0) {
		*re = 1.0;
		*im = 0.0;
	} else {
		*re = exp(-sqr(PI*xi*gG));
		*im = 0.0;
	}
}

void FTUpperAxial(double *re,double *im,double xi,double PhiA)
{
	double t1;
	if (PhiA==0.0) {
		*re = 1.0;
		*im = 0.0;
	} else {
		if (xi==0.0) {
			*re = 1.0;
			*im = 0.0;
		} else {
			t1 = sqr(sqr(PhiA*PI/180)/16/log(2)*2*PI*xi)+1;
			*re = sqrt((sqrt(t1)+1)/(2*t1));
			*im = sqrt((sqrt(t1)-1)/(2*t1));
		}
	}
}

void FTLowerAxial(double *re,double *im,double xi,double PhiA)
{
	double t1;
	if (PhiA==0.0) {
		*re = 1.0;
		*im = 0.0;
	} else {
		if (xi==0.0) {
			*re = 1.0;
			*im = 0.0;
		} else {
			t1=sqr(sqr(PhiA*PI/180)/16/log(2)*2*PI*xi)+1;
			*re = sqrt((sqrt(t1)+1)/(2*t1));
			*im = -sqrt((sqrt(t1)-1)/(2*t1));
		}
	}
}

void FTFlatSpecimen(double *re,double *im,double xi,double Phi)
/* assumes xi >= 0; the values for xi < 0 will be conjugate of that for xi > 0 */
{
	double t1;
	if (Phi==0.0) {
		*re = 1;
		*im = 0;
	} else {
		if (xi==0.0) {
			*re = 1.0;
			*im = 0.0;
		} else {
			t1=sqrt(xi)*(Phi*PI/180);
			frenel(t1,im,re);
			*re = *re/t1;
			*im = -*im/t1;
		}
	}
}

void FTTransparency(double *re,double *im,double xi,double invMuR)
{
	double t1;
	if (invMuR==0.0) {
		*re = 1;
		*im = 0;
	} else {
		if (xi==0.0) {
			*re = 1;
			*im = 0;
		} else {
			t1=2*PI*xi*invMuR;
			*re = 1/(1+t1*t1);
			*im = -t1/(1+t1*t1);
		}
	}
}

void FTReceivingSlit(double *re,double *im,double xi,double rr)
{
    double t1;
    if (rr == 0.0) {
        *re = 1;
        *im = 0;
    } else {
        if (xi==0.0) {
            *re = 1;
            *im = 0;
        } else {
            t1 = PI*xi*rr;
            *re = sin(t1)/t1;
            *im = 0;
        }
    }
}