/**************/
/* "files4.c" */
/**************/

/***** INTERFACE *****/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "typedef4.h"	/* InstrParam is defined */

#define SOURCE_FILE "data.txt"
#define PARAMETER_FILE "instr.txt"
#define DEST_FILE "deconv4.txt"
#define RESULT_FILE "result.txt"

#define DEFAULT_RHO 0.5
#define DEFAULT_X2 0.002480
#define DEFAULT_DG2 0.00006
#define DEFAULT_PHIA 2.5
#define DEFAULT_PHIE 1.0
#define DEFAULT_RS 0.15
#define DEFAULT_GONIORADIUS 185.0
#define DEFAULT_SAMPLEWIDTH 20.0
#define DEFAULT_PENETRATIONDEPTH 0.0
#define DEFAULT_INVMUR 0.0
/* #define DEFAULT_W2R 0.108108 */

/*** Global Variables ***/
extern int nData;
extern double *twot,*scat,*err;
extern double *err2;
extern InstrParam instr;
extern int EvaluateAnotherErrorIndication;

/*** Function Prototype (uses) ***/
void error(char error_text[]);	/* in "error.c" */

/*** Function Prototypes (public) ***/
void ReadParams(void);
void ReadData(void);
void SaveData(void);
void ClearData(void);

/***** IMPLEMENTATION *****/

#define PI 3.141592653589793238462643383279

static void ReadOneParam(FILE *fp, double *x, double x0,long *pl, long *pc)
{
	char *flag, buf[256];
	fpos_t pos;
	
	flag = fgets(buf,256,fp);
	fgetpos(fp,&pos);
	*pc = (long)pos;
	if (*pc - *pl > 2) {
		sscanf(buf, "%le", x);
	} else {
		*x = x0;
	}
	*pl = *pc;
}

void ReadParams(void)
{
	FILE *fp;
	fpos_t pos;
	long posl,posc;
	
	fp = fopen(PARAMETER_FILE, "rt");
	if (fp==NULL) {
		instr.g1 = 0.0;
		instr.rho = DEFAULT_RHO;
		instr.x2 = DEFAULT_X2;
		instr.dg2 = DEFAULT_DG2;
		instr.phiA = DEFAULT_PHIA;
		instr.phiE = DEFAULT_PHIE;
		instr.invmuR = DEFAULT_INVMUR;
		instr.GonioRadius = DEFAULT_GONIORADIUS;
		instr.XraySourceWidth = 0.0;
		instr.ReceivingSlitWidth = 0.0;
	} else {
		fgetpos(fp,&pos);
		posl = (long)pos;
		ReadOneParam(fp, &(instr.g1), 0.0, &posl, &posc);	/* Line 1 */
		ReadOneParam(fp, &(instr.rho), DEFAULT_RHO, &posl, &posc);	/* Line 2 */
		ReadOneParam(fp, &(instr.x2), DEFAULT_X2, &posl, &posc);	/* Line 3 */
		ReadOneParam(fp, &(instr.dg2), DEFAULT_DG2, &posl, &posc);	/* Line 4 */
		ReadOneParam(fp, &(instr.phiA), DEFAULT_PHIA, &posl, &posc);	/* Line 5 */
		ReadOneParam(fp, &(instr.phiE), DEFAULT_PHIE, &posl, &posc);	/* Line 6 */
		ReadOneParam(fp, &(instr.GonioRadius), DEFAULT_GONIORADIUS, &posl, &posc);	/* LIne 7 */
		ReadOneParam(fp, &(instr.XraySourceWidth), 0.0, &posl, &posc);	/* Line 8 */
		ReadOneParam(fp, &(instr.ReceivingSlitWidth), 0.0, &posl, &posc);	/* Line 9 */
		ReadOneParam(fp, &(instr.SampleWidth), DEFAULT_SAMPLEWIDTH, &posl, &posc);	/* Line 10 */
		ReadOneParam(fp, &(instr.PenetrationDepth), DEFAULT_PENETRATIONDEPTH, &posl, &posc);	/* Line 11 */
	}
	instr.invmuR = instr.PenetrationDepth / instr.GonioRadius;
	instr.W2R = instr.SampleWidth / instr.GonioRadius;
	instr.sintC = instr.phiE*PI/180/instr.W2R;
	if (instr.sintC > 1.0) {
		instr.twotC=180.0;
	} else {
		instr.twotC = 360/PI*asin(instr.sintC);
	}
	instr.S2R = instr.XraySourceWidth / instr.GonioRadius * 180/PI;
	instr.R2R = instr.ReceivingSlitWidth / instr.GonioRadius * 180/PI;
	instr.gGmax = 0.0;
}

void ReadData(void)
{
	FILE *fp;
	char *flag,buf[256];
	int count,cv;
	fpos_t pos;
	long posl,posc;
	double x,y,s;

	/*** First, scan the data and determine the size of arrays ***/
	fp = fopen(SOURCE_FILE,"rt");
	if (fp==NULL) {
		error("Could not open the source data file");
	}
	count = 0;
	fgetpos(fp,&pos);
	posl = (long)(pos);
	do {
		x = 0.0;
		y = 0.0;
		flag = fgets(buf,256,fp);
		fgetpos(fp,&pos);
		posc=(long)(pos);
		if (posc-posl > 2) {
			cv = sscanf(buf,"%le%le%le",&x,&y,&s);
			if (((cv ==2)||(cv==3))&&(x != 0.0)) {
				count++;
			}
		}
		posl = posc;
	} while (!feof(fp));
	fclose(fp);
	
	/*** Allocate memory for the data ***/
	twot=malloc((size_t)(count+2)*sizeof(double));
	scat=malloc((size_t)(count+2)*sizeof(double));
	err=malloc((size_t)(count+2)*sizeof(double));
	if (EvaluateAnotherErrorIndication==1) {
		err2=malloc((size_t)(count+2)*sizeof(double));
	}
	if ((twot==NULL)||(scat==NULL)||(err==NULL)||(err2==NULL)) {
		fclose(fp);
		error("Could not allocate memory for the source data");
	}

	/*** Read the data ***/
	fp = fopen(SOURCE_FILE,"rt");
	count = 0;
	fgetpos(fp,&pos);
	posl = (long)(pos);
	do {
		flag = fgets(buf,256,fp);
		fgetpos(fp,&pos);
		posc = (long)(pos);
		if (posc-posl > 2) {
			cv = sscanf(buf,"%le%le%le",&x,&y,&s);
			if (((cv ==2)||(cv==3))&&(x != 0.0)) {
				twot[count] = x;
				scat[count] = y;
				err[count] = s;
				if (EvaluateAnotherErrorIndication==1) {
					err2[count] = s;
				}
				if (cv == 2) {
					err[count] = sqrt(fabs(y));
					if (EvaluateAnotherErrorIndication==1) {
						err2[count] = err[count];
					}
				}
				count++;
			}
		}
		posl = posc;
	} while (!feof(fp));
	fclose(fp);
	nData = count;
}

void SaveData(void)
{
	FILE *fp;
	long i;

	fp = fopen(DEST_FILE,"wt");
	if (fp==NULL) {
		error("Could not open the destination data file");
	}
	for (i=0; i<nData; i++) {
		if (EvaluateAnotherErrorIndication==1) {
			fprintf(fp,"%10.4f %10.4f %10.4f %10.4f\n",
				twot[i],scat[i],err[i],err2[i]);
		} else {
			fprintf(fp,"%10.4f %10.4f %10.4f\n",
				twot[i],scat[i],err[i]);
		}
	}
	fclose(fp);
}

void ClearData(void)
{
	if (EvaluateAnotherErrorIndication==1) {
		free(err2);
	}
	free(err);
	free(scat);
	free(twot);
}
